@@ -1,11 +1,9 @@
-use kaolinite::{Document, Size, Loc};
+use kaolinite::{Document, Loc, Size};
 
 fn main() {
     // Open document with the size of 10 characters by 10 characters
-    let mut doc = Document::open(
-        Size::is(10, 10),
-        "demos/10.txt", 
-    ).expect("File couldn't be opened");
+    let mut doc =
+        Document::open(Size::is(10, 10), "demos/10.txt").expect("File couldn't be opened");
     // Load viewport
     doc.load_to(10);
     doc.move_to(&Loc { x: 0, y: 0 });
